# frozen_string_literal: true

class WIN32OLE
  def initialize(*); end

  def ConnectServer(*); end

  def Security_; end

  def ImpersonationLevel=; end

  def Model; end

  def Manufacturer; end

  def OEMStringArray; end
end
